//-----------------------------------------------------------------------------
// 
// MatrixMath :: Matrix-based Math Routines
//
// Math routines for operating on Matrices.
//
// Notes
//  (1) Removed from Euler class to allow for wider usage.
//  (2) non-class-based reoutines for conveniance.
//-----------------------------------------------------------------------------

#include "MatrixMath.h"
#include <math.h>


bool Mult( Matrix & Res, Matrix & A, Matrix & B )
{ // (Res) = (A) * (B)

    int     a_rowcount = A.RowSz();
    int     a_colcount = A.ColSz();
    int     b_rowcount = B.RowSz();
    int     b_colcount = B.ColSz();

    int     res_rc = a_rowcount;
    int     res_cc = b_colcount;

    float   sum;

    if ( a_colcount != b_rowcount )
        return false;                   // operation cannot be performed, row or col mis-match

  // Check/correct Res vector

    if ( (Res.RowSz() != res_rc) || (Res.ColSz() != res_cc) )
    { // re-dimension result vector

        Res.dim( res_rc, res_cc );
    }

    for ( int r = 0 ; r < res_rc ; r++ )
    {
        for ( int c = 0 ; c < res_cc ; c++ )
        {
            sum = 0;
            for ( int s = 0 ; s < a_colcount ; s++ )
            {
                sum += ( A.val(r,s) * B.val(s,c) );
            }
            Res.val(r,c) = sum;
        }
    }

    return true;
}


bool Mult( Matrix & Res, Matrix & A, float b )
{ // (Res) = (A) * (b)

    int rowcount = A.RowSz();
    int colcount = A.ColSz();

  // Correct Res, if required.

    if ( (Res.RowSz() != rowcount) || (Res.ColSz() != colcount) )
    { // re-dimension result vector

        Res.dim( rowcount, colcount );
    }

    for ( int r = 0 ; r < rowcount ; r++ )
        for ( int c = 0 ; c < colcount ; c++ )
            Res.val(r,c) = A.val(r,c) * b;

    return true;
}


bool Add( Matrix & Res, Matrix & A, Matrix & B )
{ // (Res) = (A) + (B)

    if ( (Res.RowSz() != A.RowSz()) ||
         (Res.RowSz() != B.RowSz()) ||
         (Res.ColSz() != A.ColSz()) ||
         (Res.ColSz() != B.ColSz()) )
    {
        return false;
    }

    for ( int r = 0 ; r < Res.RowSz() ; r++ )
        for ( int c = 0 ; c < Res.ColSz() ; c++ )
            Res.val(r,c) = A.val(r,c) + B.val(r,c);

    return true;
}


bool Sub( Matrix & Res, Matrix & A, Matrix & B )
{ // (Res) = (A) - (B)

    if ( (Res.RowSz() != A.RowSz()) ||
         (Res.RowSz() != B.RowSz()) ||
         (Res.ColSz() != A.ColSz()) ||
         (Res.ColSz() != B.ColSz()) )
    {
        return false;
    }

    for ( int r = 0 ; r < Res.RowSz() ; r++ )
        for ( int c = 0 ; c < Res.ColSz() ; c++ )
            Res.val(r,c) = A.val(r,c) - B.val(r,c);

    return true;
}


float AMag( Matrix & Ap )
{
    return sqrt( (Ap.val(ROW_X,0) * Ap.val(ROW_X,0)) + 
                 (Ap.val(ROW_Y,0) * Ap.val(ROW_Y,0)) + 
                 (Ap.val(ROW_Z,0) * Ap.val(ROW_Z,0)) 
               );
}

