//-----------------------------------------------------------------------------
// 
// MatrixMath :: Matrix-based Math Routines
//
// Math routines for operating on Matrices.
//
// Notes
//  (1) Removed from Euler class to allow for wider usage.
//  (2) non-class-based reoutines for conveniance.
//-----------------------------------------------------------------------------

#ifndef MATRIXMATH_H
#define MATRIXMATH_H

#include "Matrix.h"

// Matrix Multiply :: (Res) = (A) * (B)
// ---------------------------------------------------------------------------
// Operator   : MULTIPLY
// Parameters : all params are matrices, order is important!
// ===========================================================================
bool    Mult( Matrix & Res, Matrix & A, Matrix & B );

// Matrix Multiply :: (Res) = (A) * (b)
// ---------------------------------------------------------------------------
// Operator   : MULTIPLY
// Parameters : Matrix and scalar
// ===========================================================================
bool    Mult( Matrix & Res, Matrix & A, float b );

// Matrix Addition :: (Res) = (A) + (B)
// ---------------------------------------------------------------------------
// Operator   : ADDITION
// Parameters : all params are matrices
// ===========================================================================
bool    Add( Matrix & Res, Matrix & A, Matrix & B );

// Matrix Subtraction :: (Res) = (A) - (B)
// ---------------------------------------------------------------------------
// Operator   : SUBTRACTION
// Parameters : all params are matrices
// ===========================================================================
bool    Sub( Matrix & Res, Matrix & A, Matrix & B );

// Compute the total magnitude of a (3x1) vector
// ---------------------------------------------------------------------------
// Operator   : MAGNITUDE
// Parameters : computes the length of vector in Ap, from origon. Returns a
//              scalar value.
// ===========================================================================
float   AMag( Matrix & Ap );


#endif // MATRIXMATH_H

